@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2022-2023. All Rights Reserved.
+ * Copyright Ericsson AB 2022-2025. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,28 +33,6 @@
 #include "socket_dbg.h"
 
 
-/* ********************************************************************* *
- *                              SOCKET and HANDLE                        *
- * ********************************************************************* *
- */
-
-#if defined(__WIN32__)
-
-#define INVALID_EVENT NULL
-#define SOCKET_FORMAT_STR "%lld"
-
-#else
-
-#define INVALID_HANDLE (-1)
-typedef int HANDLE;
-#define INVALID_SOCKET (-1)
-typedef int SOCKET; /* A subset of HANDLE */
-#define INVALID_EVENT INVALID_HANDLE
-#define SOCKET_FORMAT_STR "%d"
-
-#endif
-
-
 /* ********************************************************************* *
  *                   Socket state defs and macros                        *
  * ********************************************************************* *
@@ -361,7 +339,8 @@ typedef struct {
     int          iov_max;
 
     /* XXX
-     * Should be locked but too awkward for no gain since it is not used yet
+     * Should be locked but too awkward for no gain
+     * since it is not used yet.
      */
     BOOLEAN_T    iow; // Where do we send this? Subscription?
 
