@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2003-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -28,7 +28,8 @@
 	 init_per_testcase/2,end_per_testcase/2,
 	 errors/1,record_test_2/1,record_test_3/1,record_access_in_guards/1,
 	 guard_opt/1,eval_once/1,foobar/1,missing_test_heap/1,
-	 nested_access/1,coverage/1,grab_bag/1,slow_compilation/1]).
+	 nested_access/1,coverage/1,grab_bag/1,slow_compilation/1,
+         duplicate_update_record/1]).
 
 init_per_testcase(_Case, Config) ->
     Config.
@@ -48,7 +49,8 @@ groups() ->
       [errors,record_test_2,record_test_3,
        record_access_in_guards,guard_opt,eval_once,foobar,
        missing_test_heap,nested_access,coverage,grab_bag,
-       slow_compilation]}].
+       slow_compilation,
+       duplicate_update_record]}].
 
 
 init_per_suite(Config) ->
@@ -804,6 +806,20 @@ slow_compilation(Config) when is_list(Config) ->
      {f56,R#slow_r.f56},{f57,R#slow_r.f57},{f58,R#slow_r.f58},
      {f59,R#slow_r.f59}].
 
+%% GH-8783: Duplicate indexes in update_record crashed the emulator.
+duplicate_update_record(Config) when is_list(Config) ->
+    DuplicateUR0 = id({id(left), id(right)}),
+    {_, _} = DuplicateUR0,
+
+    DuplicateUR1 = erlang:setelement(2, DuplicateUR0, false),
+    DuplicateUR = erlang:setelement(2, DuplicateUR1, false),
+    {'EXIT', _} = catch duplicate_update_record_1(DuplicateUR),
+
+    ok.
+
+duplicate_update_record_1(_) ->
+    erlang:error(crash).
+
 first_arg(First, _) -> First.
 
 id(I) -> I.
