@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2017-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2017-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -35,10 +35,8 @@
          end_per_testcase/2,
          get_all_possible_methods/0,
          get_all_possible_methods/1,
-         engine_load_all_methods/0,
-         engine_load_all_methods/1,
-         engine_load_some_methods/0,
-         engine_load_some_methods/1,
+         engine_load_register_method/0,
+         engine_load_register_method/1,
          engine_list/0,
          engine_list/1,
          get_id_and_name/0,
@@ -61,10 +59,13 @@
          ctrl_cmd_string_optional/1,
          ensure_load/0,
          ensure_load/1,
+	 gc_clean/0,
+	 gc_clean/1,
          sign_verify_rsa/1,
          sign_verify_rsa_fake/1,
          sign_verify_dsa/1,
          sign_verify_ecdsa/1,
+         sign_verify_eddsa/1,
          sign_verify_rsa_pwd/1,
          sign_verify_rsa_pwd_bad_pwd/1,
          priv_encrypt_pub_decrypt_rsa/1,
@@ -90,8 +91,7 @@ suite() ->
 all() ->
     [
      get_all_possible_methods,
-     engine_load_all_methods,
-     engine_load_some_methods,
+     engine_load_register_method,
      engine_list,
      get_id_and_name,
      engine_by_id,
@@ -103,6 +103,7 @@ all() ->
      ctrl_cmd_string,
      ctrl_cmd_string_optional,
      ensure_load,
+     gc_clean,
      {group, engine_stored_key},
      {group, engine_fakes_rsa}
     ].
@@ -113,6 +114,7 @@ groups() ->
        sign_verify_rsa,
        sign_verify_dsa,
        sign_verify_ecdsa,
+       sign_verify_eddsa,
        sign_verify_rsa_pwd,
        sign_verify_rsa_pwd_bad_pwd,
        priv_encrypt_pub_decrypt_rsa,
@@ -126,26 +128,31 @@ groups() ->
        get_pub_from_priv_key_dsa,
        get_pub_from_priv_key_ecdsa
       ]},
-    {engine_fakes_rsa, [], [sign_verify_rsa_fake
-                     ]}
-     ].
+    {engine_fakes_rsa, [],
+     [
+      sign_verify_rsa_fake
+     ]}
+    ].
 
 
 init_per_suite(Config) ->
-    try {os:type(), crypto:info_lib()} of
-        {_, [{_,_, <<"OpenSSL 1.0.1s-freebsd  1 Mar 2016">>}]} ->
+    try {engine_supported(), os:type(), crypto:info_lib()} of
+        {false,_,[{_,_,OpenSSLversion}]} ->
+            {skip, "Engine not supported by " ++ OpenSSLversion};
+
+        {_,_, [{_,_, <<"OpenSSL 1.0.1s-freebsd  1 Mar 2016">>}]} ->
             {skip, "Problem with engine on OpenSSL 1.0.1s-freebsd"};
 
-        {_, [{_,_,<<"LibreSSL 2.1.",_/binary>>}]} ->
+        {_,_, [{_,_,<<"LibreSSL 2.1.",_/binary>>}]} ->
             {skip, "Problem with engine on older LibreSSL 2.1.*"};
 
-        {{unix,darwin}, _} ->
+        {_,{unix,darwin}, _} ->
             {skip, "Engine unsupported on Darwin"};
-        
-        {{win32,_}, _} ->
+
+        {_,{win32,_}, _} ->
             {skip, "Engine unsupported on Windows"};
-        
-        {OS, Res} ->
+
+        {_,OS, Res} ->
             ct:log("crypto:info_lib() -> ~p\nos:type() -> ~p", [Res,OS]),
             try crypto:start() of
                 ok ->
@@ -162,6 +169,11 @@ init_per_suite(Config) ->
 end_per_suite(_Config) ->
     ok.
 
+engine_supported() ->
+    try crypto:engine_list(), true
+    catch error:notsup -> false
+    end.
+
 %%--------------------------------------------------------------------
 init_per_group(engine_stored_key, Config) ->
     group_load_engine(Config,  [engine_method_rsa]);
@@ -184,10 +196,10 @@ init_per_group(engine_fakes_rsa, Config) ->
 init_per_group(_Group, Config0) ->
     Config0.
 
-
 group_load_engine(Config, ExcludeMthds) ->
-    case load_storage_engine(Config, ExcludeMthds) of
+    case load_storage_engine(Config) of
         {ok, E} ->
+	    ok = crypto:engine_register(E, crypto:engine_get_all_methods() -- ExcludeMthds),
             KeyDir = key_dir(Config),
             ct:log("storage engine ~p loaded.~nKeyDir = ~p", [E,KeyDir]),
             [{storage_engine,E}, {storage_dir,KeyDir} | Config];
@@ -202,10 +214,6 @@ group_load_engine(Config, ExcludeMthds) ->
             {fail, "Engine load failed"}
     end.
 
-
-
-
-
 end_per_group(_, Config) ->
     case proplists:get_value(storage_engine, Config) of
         undefined ->
@@ -218,10 +226,9 @@ end_per_group(_, Config) ->
 init_per_testcase(Case, Config) ->
     HasMD5 = lists:member(md5, crypto:supports(hashs)),
     case Case of
-        multiple_engine_load     when HasMD5==false -> {skip, "md5 not available"};
-        ensure_load              when HasMD5==false -> {skip, "md5 not available"};
-        engine_load_some_methods when HasMD5==false -> {skip, "md5 not available"};
-        engine_load_all_methods  when HasMD5==false -> {skip, "md5 not available"};
+        ensure_load                 when HasMD5==false -> {skip, "md5 not available"};
+	engine_load_register_method when HasMD5==false -> {skip, "md5 not available"};
+	gc_clean                    when HasMD5==false -> {skip, "md5 not available"};
         _ ->
             case string:tokens(atom_to_list(Case),"_") of
                 ["sign","verify",Type|_] ->
@@ -255,12 +262,13 @@ get_all_possible_methods(Config) when is_list(Config) ->
             {skip, "Engine not supported on this SSL version"}
     end.
 
-engine_load_all_methods()->
+%%-------------------------------------------------------------------------
+engine_load_register_method()->
     [{doc, "Use a dummy md5 engine that does not implement md5"
       "but rather returns a static binary to test that crypto:engine_load "
       "functions works."}].
 
-engine_load_all_methods(Config) when is_list(Config) ->
+engine_load_register_method(Config) when is_list(Config) ->
     case crypto:get_test_engine() of
         {error, notexist} ->
             {skip, "OTP Test engine not found"};
@@ -272,8 +280,10 @@ engine_load_all_methods(Config) when is_list(Config) ->
                 case crypto:engine_load(<<"dynamic">>,
                                         [{<<"SO_PATH">>, Engine},
                                          <<"LOAD">>],
-                                        []) of
+                                        []) of		    
                     {ok, E} ->
+			ok = crypto:engine_register(E, [engine_method_digests]),
+
                         case crypto:hash(md5, "Don't panic") of
                             Md5Hash1 ->
                                 ct:fail(fail_to_load_still_original_engine);
@@ -282,7 +292,10 @@ engine_load_all_methods(Config) when is_list(Config) ->
                             _ ->
                                 ct:fail(fail_to_load_engine)
                         end,
+
+			ok = crypto:engine_unregister(E, [engine_method_digests]),
                         ok = crypto:engine_unload(E),
+
                         case crypto:hash(md5, "Don't panic") of
                             Md5Hash2 ->
                                 ct:fail(fail_to_unload_still_test_engine);
@@ -300,56 +313,7 @@ engine_load_all_methods(Config) when is_list(Config) ->
            end
     end.
 
-engine_load_some_methods()->
-    [{doc, "Use a dummy md5 engine that does not implement md5"
-      "but rather returns a static binary to test that crypto:engine_load "
-      "functions works."}].
-
-engine_load_some_methods(Config) when is_list(Config) ->
-    case crypto:get_test_engine() of
-        {error, notexist} ->
-            {skip, "OTP Test engine not found"};
-        {ok, Engine} ->
-            try
-                Md5Hash1 =  <<106,30,3,246,166,222,229,158,244,217,241,179,50,232,107,109>>,
-                Md5Hash1 = crypto:hash(md5, "Don't panic"),
-                Md5Hash2 =  <<0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15>>,
-                EngineMethods = crypto:engine_get_all_methods() --
-                    [engine_method_dh, engine_method_rand,
-                     engine_method_ciphers, engine_method_store,
-                     engine_method_pkey_meths, engine_method_pkey_asn1_meths],
-                case crypto:engine_load(<<"dynamic">>,
-                                        [{<<"SO_PATH">>, Engine},
-                                         <<"LOAD">>],
-                                        [],
-                                        EngineMethods) of
-                    {ok, E} ->
-                        case crypto:hash(md5, "Don't panic") of
-                            Md5Hash1 ->
-                                ct:fail(fail_to_load_engine_still_original);
-                            Md5Hash2 ->
-                                ok;
-                            _ ->
-                                ct:fail(fail_to_load_engine)
-                        end,
-                        ok = crypto:engine_unload(E),
-                        case crypto:hash(md5, "Don't panic") of
-                            Md5Hash2 ->
-                                ct:fail(fail_to_unload_still_test_engine);
-                            Md5Hash1 ->
-                                ok;
-                            _ ->
-                                ct:fail(fail_to_unload_engine)
-                        end;
-                    {error, bad_engine_id} ->
-                    {skip, "Dynamic Engine not supported"}
-                end
-           catch
-               error:notsup ->
-                  {skip, "Engine not supported on this SSL version"}
-           end
-    end.
-
+%%-------------------------------------------------------------------------
 engine_list()->
     [{doc, "Test add and remove engine ID to the SSL internal engine list."}].
 
@@ -380,6 +344,7 @@ engine_list(Config) when is_list(Config) ->
            end
     end.
 
+%%-------------------------------------------------------------------------
 get_id_and_name()->
     [{doc, "Test fetching id and name from an engine."}].
 
@@ -406,6 +371,7 @@ get_id_and_name(Config) when is_list(Config) ->
            end
     end.
 
+%%-------------------------------------------------------------------------
 engine_by_id()->
     [{doc, "Test fetching a new reference the the engine when the"
      "engine id is added to the SSL engine list."}].
@@ -499,6 +465,7 @@ bad_arguments(Config) when is_list(Config) ->
           end
     end.
 
+%%-------------------------------------------------------------------------
 unknown_engine() ->
     [{doc, "Try to load a non existent engine."}].
 
@@ -511,6 +478,7 @@ unknown_engine(Config) when is_list(Config) ->
            {skip, "Engine not supported on this SSL version"}
     end.
 
+%%-------------------------------------------------------------------------
 pre_command_fail_bad_value() ->
     [{doc, "Test pre command due to bad value"}].
 
@@ -533,6 +501,7 @@ pre_command_fail_bad_value(Config) when is_list(Config) ->
            {skip, "Engine not supported on this SSL version"}
     end.
 
+%%-------------------------------------------------------------------------
 pre_command_fail_bad_key() ->
     [{doc, "Test pre command due to bad key"}].
 
@@ -558,6 +527,7 @@ pre_command_fail_bad_key(Config) when is_list(Config) ->
           {skip, "Engine not supported on this SSL version"}
    end.
 
+%%-------------------------------------------------------------------------
 failed_engine_init()->
     [{doc, "Test failing engine init due to missed pre command"}].
 
@@ -582,7 +552,6 @@ failed_engine_init(Config) when is_list(Config) ->
           {skip, "Engine not supported on this SSL version"}
    end.
 
-
 %%-------------------------------------------------------------------------
 %% Test the optional flag in ctrl comands
 ctrl_cmd_string()->
@@ -615,6 +584,7 @@ ctrl_cmd_string(Config) when is_list(Config) ->
           {skip, "Engine not supported on this SSL version"}
    end.
 
+%%-------------------------------------------------------------------------
 ctrl_cmd_string_optional()->
     [{doc, "Test that a not known optional ctrl command do not fail"}].
 ctrl_cmd_string_optional(Config) when is_list(Config) ->
@@ -647,6 +617,8 @@ ctrl_cmd_string_optional(Config) when is_list(Config) ->
    end.
 
 %%-------------------------------------------------------------------------
+%
+%
 ensure_load()->
     [{doc, "Test the special ensure load function."}].
 
@@ -661,6 +633,7 @@ ensure_load(Config) when is_list(Config) ->
                 Md5Hash2 =  <<0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15>>,
                 case crypto:ensure_engine_loaded(<<"MD5">>, Engine) of
                     {ok, E} ->
+			ok = crypto:engine_register(E, [engine_method_digests]),
 
                         case crypto:hash(md5, "Don't panic") of
                             Md5Hash1 ->
@@ -715,13 +688,14 @@ ensure_load(Config) when is_list(Config) ->
                         ok = crypto:ensure_engine_unloaded(E),
                         case crypto:hash(md5, "Don't panic") of
                             Md5Hash1 ->
-                                ok;
+                                ct:fail(fail_test_engine_not_loaded);
                             Md5Hash2 ->
-                                ct:fail(fail_to_unload_still_test_engine);
+                                ok;
                             _ ->
                                 ct:fail(fail_to_unload_engine)
                         end,
 
+			ok = crypto:engine_unregister(E3, [engine_method_digests]),
 			ok = crypto:ensure_engine_unloaded(E3),
                         case crypto:hash(md5, "Don't panic") of
                             Md5Hash1 ->
@@ -741,6 +715,85 @@ ensure_load(Config) when is_list(Config) ->
            end
     end.
 
+%%-------------------------------------------------------------------------
+gc_clean()->
+    [{doc, "Test the special ensure load function."}].
+
+gc_clean(Config) when is_list(Config) ->
+    case crypto:get_test_engine() of
+        {error, notexist} ->
+            {skip, "OTP Test engine not found"};
+        {ok, Engine} ->
+
+	    Md5Hash1 = <<106,30,3,246,166,222,229,158,244,217,241,179,50,232,107,109>>,
+	    Md5Hash1 = crypto:hash(md5, "Don't panic"),
+	    Md5Hash2 =  <<0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15>>,
+
+	    load_without_unload(Engine, Md5Hash1, Md5Hash2),
+
+	    erlang:garbage_collect(),
+	    timer:sleep(1000),
+
+	    case crypto:hash(md5, "Don't panic") of
+		Md5Hash2 ->
+		    ct:fail(fail_to_unload_still_test_engine);
+		Md5Hash1 ->
+		    ok;
+		_ ->
+		    ct:fail(fail_to_unload_engine)
+	    end,
+	    
+	    try
+		case crypto:ensure_engine_loaded(<<"MD5">>, Engine) of
+		    {ok, E} ->
+			ok = crypto:engine_register(E, [engine_method_digests]),
+			case crypto:hash(md5, "Don't panic") of
+			    Md5Hash1 ->
+				ct:fail(fail_to_load_still_original_engine);
+			    Md5Hash2 ->
+				ok;
+			    _ ->
+				ct:fail(fail_to_load_engine)
+			end,
+			
+			ok = crypto:engine_unregister(E, [engine_method_digests]),
+			ok = crypto:engine_remove(E),
+			ok = crypto:ensure_engine_unloaded(E),
+			
+			case crypto:hash(md5, "Don't panic") of
+			    Md5Hash2 ->
+				ct:fail(fail_to_unload_still_test_engine);
+			    Md5Hash1 ->
+				ok;
+			    _ ->
+				ct:fail(fail_to_unload_engine)
+			end;
+		    {error, bad_engine_id} ->
+                        {skip, "Dynamic Engine not supported"}
+		end
+	     catch
+		 error:notsup ->
+			     {skip, "Engine not supported on this SSL version"}
+	     end
+    end.
+    
+load_without_unload(Engine, Md5Hash1, Md5Hash2) ->
+    case crypto:ensure_engine_loaded(<<"MD5">>, Engine) of
+	{ok, E} ->
+	    ok = crypto:engine_register(E, [engine_method_digests]),
+	    case crypto:hash(md5, "Don't panic") of
+		Md5Hash1 ->
+		    ct:fail(fail_to_load_still_original_engine);
+		Md5Hash2 ->
+		    ok = crypto:engine_unregister(E, [engine_method_digests]),
+		    ok;
+		_ ->
+		    ct:fail(fail_to_load_engine)
+	    end;
+	 {error, bad_engine_id} ->
+	    io:format("load_without_unload: bad_engine_id\n", [])
+    end.
+
 %%%----------------------------------------------------------------
 %%% Pub/priv key storage tests.  Those are for testing the crypto.erl
 %%% support for using priv/pub keys stored in an engine.
@@ -774,6 +827,13 @@ sign_verify_ecdsa(Config) ->
              key_id => key_id(Config, "ecdsa_public_key.pem")},
     sign_verify(ecdsa, sha, Priv, Pub).
 
+sign_verify_eddsa(Config) ->
+    Priv = #{engine => engine_ref(Config),
+             key_id => key_id(Config, "eddsa_private_key.pem")},
+    Pub  = #{engine => engine_ref(Config),
+             key_id => key_id(Config, "eddsa_public_key.pem")},
+    sign_verify(eddsa, sha, Priv, Pub).
+
 sign_verify_rsa_pwd(Config) ->
     Priv = #{engine => engine_ref(Config),
              key_id => key_id(Config, "rsa_private_key_pwd.pem"),
@@ -932,16 +992,14 @@ pkey_supported(Type) ->
     lists:member(Type, proplists:get_value(public_keys, crypto:supports(), [])).
 
 
-load_storage_engine(_Config, ExcludeMthds) ->
+load_storage_engine(_Config) ->
     case crypto:get_test_engine() of
-        {ok, Engine} ->
+        {ok, EngineLibPath} ->
             try crypto:engine_load(<<"dynamic">>,
-                                   [{<<"SO_PATH">>, Engine},
-                                    <<"LOAD">>],
-                                   [],
-                                   crypto:engine_get_all_methods() -- ExcludeMthds
-                                  )
-            catch
+				   [{<<"SO_PATH">>, EngineLibPath},
+				    <<"LOAD">>],
+				   [])
+	    catch
                 error:notsup ->
                     {error, notsup}
             end;
